@@ -3,16 +3,15 @@ import inspect
 import boto.connection
 
 from ddtrace import config
+from ddtrace.constants import ANALYTICS_SAMPLE_RATE_KEY
+from ddtrace.constants import SPAN_MEASURED_KEY
+from ddtrace.ext import SpanTypes
+from ddtrace.ext import aws
+from ddtrace.ext import http
+from ddtrace.pin import Pin
+from ddtrace.utils.wrappers import unwrap
 from ddtrace.vendor import wrapt
 
-from ...constants import ANALYTICS_SAMPLE_RATE_KEY
-from ...constants import SPAN_MEASURED_KEY
-from ...ext import SpanTypes
-from ...ext import aws
-from ...ext import http
-from ...pin import Pin
-from ...utils.wrappers import unwrap
-
 
 # Original boto client class
 _Boto_client = boto.connection.AWSQueryConnection
@@ -38,7 +37,7 @@ def patch():
 
     # AWSQueryConnection and AWSAuthConnection are two different classes called by
     # different services for connection.
-    # For exemple EC2 uses AWSQueryConnection and S3 uses AWSAuthConnection
+    # For example EC2 uses AWSQueryConnection and S3 uses AWSAuthConnection
     wrapt.wrap_function_wrapper("boto.connection", "AWSQueryConnection.make_request", patched_query_request)
     wrapt.wrap_function_wrapper("boto.connection", "AWSAuthConnection.make_request", patched_auth_request)
     Pin(service="aws", app="aws").onto(boto.connection.AWSQueryConnection)
