@@ -3,15 +3,16 @@ from __future__ import absolute_import
 import os.path
 import sys
 import threading
-from typing import Optional
-from typing import Tuple
+import typing
 
 import attr
-from six.moves import _thread
 
 from ddtrace.internal import compat
+from ddtrace.internal import nogevent
 from ddtrace.profiling import collector
 from ddtrace.profiling import event
+from ddtrace.profiling.collector import _task
+from ddtrace.profiling.collector import _threading
 from ddtrace.profiling.collector import _traceback
 from ddtrace.utils import attr as attr_utils
 from ddtrace.vendor import wrapt
@@ -40,16 +41,9 @@ class LockReleaseEvent(LockEventBase):
 
 
 def _current_thread():
-    # This is a custom version of `threading.current_thread`
-    # that does not try # to create a `DummyThread` on `KeyError`.
-    ident = _thread.get_ident()
-    try:
-        thread = threading._active[ident]
-    except KeyError:
-        name = None
-    else:
-        name = thread.name
-    return ident, name
+    # type: (...) -> typing.Tuple[int, str]
+    thread_id = nogevent.thread_get_ident()
+    return thread_id, _threading.get_thread_name(thread_id)
 
 
 # We need to know if wrapt is compiled in C or not. If it's not using the C module, then the wrappers function will
@@ -77,26 +71,6 @@ class _ProfiledLock(wrapt.ObjectProxy):
         code = frame.f_code
         self._self_name = "%s:%d" % (os.path.basename(code.co_filename), frame.f_lineno)
 
-    def _get_trace_and_span_info(self):
-        # type: (...) -> Tuple[Optional[int], Optional[int], Optional[str], Optional[str]]
-        """Return current trace id, span id and trace resource and type."""
-        if self._self_tracer is None:
-            return (None, None, None, None)
-
-        ctxt = self._self_tracer.current_trace_context()
-        if ctxt is None:
-            return (None, None, None, None)
-
-        root = self._self_tracer.current_root_span()
-        if root is None:
-            resource = None
-            span_type = None
-        else:
-            resource = root.resource
-            span_type = root.span_type
-
-        return (ctxt.trace_id, ctxt.span_id, resource, span_type)
-
     def acquire(self, *args, **kwargs):
         if not self._self_capture_sampler.capture():
             return self.__wrapped__.acquire(*args, **kwargs)
@@ -109,22 +83,23 @@ class _ProfiledLock(wrapt.ObjectProxy):
                 end = self._self_acquired_at = compat.monotonic_ns()
                 thread_id, thread_name = _current_thread()
                 frames, nframes = _traceback.pyframe_to_frames(sys._getframe(1), self._self_max_nframes)
-                trace_id, span_id, trace_resource, trace_type = self._get_trace_and_span_info()
-                self._self_recorder.push_event(
-                    LockAcquireEvent(
-                        lock_name=self._self_name,
-                        frames=frames,
-                        nframes=nframes,
-                        thread_id=thread_id,
-                        thread_name=thread_name,
-                        trace_id=trace_id,
-                        span_id=span_id,
-                        trace_resource=trace_resource,
-                        trace_type=trace_type,
-                        wait_time_ns=end - start,
-                        sampling_pct=self._self_capture_sampler.capture_pct,
-                    )
+                task_id, task_name = _task.get_task(thread_id)
+                event = LockAcquireEvent(
+                    lock_name=self._self_name,
+                    frames=frames,
+                    nframes=nframes,
+                    thread_id=thread_id,
+                    thread_name=thread_name,
+                    task_id=task_id,
+                    task_name=task_name,
+                    wait_time_ns=end - start,
+                    sampling_pct=self._self_capture_sampler.capture_pct,
                 )
+
+                if self._self_tracer is not None:
+                    event.set_trace_info(self._self_tracer.current_span())
+
+                self._self_recorder.push_event(event)
             except Exception:
                 pass
 
@@ -138,22 +113,23 @@ class _ProfiledLock(wrapt.ObjectProxy):
                         end = compat.monotonic_ns()
                         frames, nframes = _traceback.pyframe_to_frames(sys._getframe(1), self._self_max_nframes)
                         thread_id, thread_name = _current_thread()
-                        trace_id, span_id, trace_resource, trace_type = self._get_trace_and_span_info()
-                        self._self_recorder.push_event(
-                            LockReleaseEvent(
-                                lock_name=self._self_name,
-                                frames=frames,
-                                nframes=nframes,
-                                thread_id=thread_id,
-                                thread_name=thread_name,
-                                trace_id=trace_id,
-                                span_id=span_id,
-                                trace_resource=trace_resource,
-                                trace_type=trace_type,
-                                locked_for_ns=end - self._self_acquired_at,
-                                sampling_pct=self._self_capture_sampler.capture_pct,
-                            )
+                        task_id, task_name = _task.get_task(thread_id)
+                        event = LockReleaseEvent(
+                            lock_name=self._self_name,
+                            frames=frames,
+                            nframes=nframes,
+                            thread_id=thread_id,
+                            thread_name=thread_name,
+                            task_id=task_id,
+                            task_name=task_name,
+                            locked_for_ns=end - self._self_acquired_at,
+                            sampling_pct=self._self_capture_sampler.capture_pct,
                         )
+
+                        if self._self_tracer is not None:
+                            event.set_trace_info(self._self_tracer.current_span())
+
+                        self._self_recorder.push_event(event)
                     finally:
                         del self._self_acquired_at
             except Exception:
