@@ -17,6 +17,7 @@ from typing import Union
 
 from ddtrace import config
 from ddtrace.filters import TraceFilter
+from ddtrace.utils.deprecation import deprecation
 from ddtrace.vendor import debtcollector
 
 from . import _hooks
@@ -143,10 +144,21 @@ class Tracer(object):
                 sync_mode=self._use_sync_mode(),
             )
         self.writer = writer  # type: TraceWriter
-        self._partial_flush_enabled = asbool(get_env("tracer", "partial_flush_enabled", default=False))
+
+        # DD_TRACER_... should be deprecated after version 1.0.0 is released
+        pfe_default_value = False
+        pfms_default_value = 500
+        if "DD_TRACER_PARTIAL_FLUSH_ENABLED" in os.environ or "DD_TRACER_PARTIAL_FLUSH_MIN_SPANS" in os.environ:
+            deprecation("DD_TRACER_... use DD_TRACE_... instead", version="1.0.0")
+            pfe_default_value = asbool(get_env("tracer", "partial_flush_enabled", default=pfe_default_value))
+            pfms_default_value = int(
+                get_env("tracer", "partial_flush_min_spans", default=pfms_default_value)  # type: ignore[arg-type]
+            )
+        self._partial_flush_enabled = asbool(get_env("trace", "partial_flush_enabled", default=pfe_default_value))
         self._partial_flush_min_spans = int(
-            get_env("tracer", "partial_flush_min_spans", default=500)  # type: ignore[arg-type]
+            get_env("trace", "partial_flush_min_spans", default=pfms_default_value)  # type: ignore[arg-type]
         )
+
         self._initialize_span_processors()
         self._hooks = _hooks.Hooks()
         atexit.register(self._atexit)
@@ -246,6 +258,25 @@ class Tracer(object):
             return active.context
         return None
 
+    def get_log_correlation_context(self):
+        # type: () -> Dict[str, str]
+        """Retrieves the data used to correlate a log with the current active trace.
+        Generates a dictionary for custom logging instrumentation including the trace id and
+        span id of the current active span, as well as the configured service, version, and environment names.
+        If there is no active span, a dictionary with an empty string for each value will be returned.
+        """
+        span = None
+        if self.enabled:
+            span = self.current_span()
+
+        return {
+            "trace_id": str(span.trace_id) if span else "0",
+            "span_id": str(span.span_id) if span else "0",
+            "service": config.service or "",
+            "version": config.version or "",
+            "env": config.env or "",
+        }
+
     # TODO: deprecate this method and make sure users create a new tracer if they need different parameters
     def configure(
         self,
